/*
Remplacer une couleur via la caméra
------------------------------------

p5js permet facilement d'accéder à la caméra sur mobile ou via une webcam

par contre ça nécessite l'extension p5dom,
qu'on trouve dans le pack p5js.complete ici :
http://p5js.org/download/

ensuite l'idée c'est d'aller lire la valeur colorée de chaque pixel de l'image filmée
et de modifier ces valeurs...

puis d'afficher l'image altérée.
*/

// settings

var couleurCible = [150, 160, 190]; // je vise le bleu d'une couv de comix
var replaceColor = [255, 255, 255]; // la couleur de remplacement
var marge = 50; // la tolérance autour de cette couleur (entre 0 et 255, en gros)

function setup(){ 

	createCanvas(windowWidth, windowHeight);

	// on capture la camera grâce à p5dom
	capture = createCapture(VIDEO);
	capture.size(320, 240); // taille d'échantillonnage, pas forcément celle de l'affichage
}
function draw(){
	
	// écraser l'image brute de la cam affichée par défaut
	noStroke();
	fill(255);
	rect(0, 0, width, height);

	// ensuite on affiche cette capture comme n'importe quelle image
	image(capture, 60, 60, 320, 240);


	// on charge les valeurs de tous le spixels du sketch
	loadPixels(); // -> remplit la variable pixels[]

	// nous on n'est intéressés que par la partie "image de la cam"
	var startat = {x: 60, y:60};
	var endat = {x: 60+320, y: 60+240};

	// on va lire les valeurs des pixels en question
	var d = pixelDensity();
	for (var i = (startat.x+width*startat.y); i <= (endat.x+width*endat.y); i++) {
		
		// on récupère le numéro du pixel x/y ciblé
		var num = (i*4*d); // 4*d... cherchez pas, c'est ça qu'io faut faire :)

		/*
		pixels[num] contient la valeur R
		pixels[num+1] contient la valeur V
		pixels[num+2] contient la valeur B
		pixels[num+3] contient la valeur A (alpha)
		du pixel qu'on voulait cibler !
		*/

		// ici on peut imaginer plein de traitements possibles du pixel ciblé
		// en modifiant les couleurs du pixel en question

		// on calcule la "distance" avec la couleur ciblée...
		var distR = pixels[num]-couleurCible[0];
		var distV = pixels[num+1]-couleurCible[1];
		var distB = pixels[num+2]-couleurCible[2];

		// et si c'est proche...
		if(distR<marge && distR>-marge && distV<marge && distV>-marge && distB<marge && distB>-marge){

			// ... on remplace par la couleur indiquée plus haut !
			pixels[num] = replaceColor[0];
			pixels[num+1] = replaceColor[1];
			pixels[num+2] = replaceColor[2];
		}

		/*
		bon c'est pas parfait, déjà il faut bien choisir sa couleur "moyenne"
		on peut aussi comparer sur R et V seulement par exemple...
		ou V & B, à tester !
		*/
	};

	// et pour répercuter les modifs apportées à la grille de pixels :
	updatePixels();

	// afficher le titre du sketch
	noStroke();
	fill(255); // rectangle blanc sous le texte...
	rect(15, 8, 230, 34)
	fill(0);
	text("// Remplacer une couleur via la caméra", 30, 30);

}

// accessoirement voici un color picker, pour afficher dans la console
// la valeur colorée d'un pixel... ça peut aider
// cliquez dans l'image et regardez dans la console !
function mouseReleased(){
	var c = get(mouseX, mouseY);
	console.log('R:'+c[0]+' V:'+c[1]+' B:'+c[2]+' A:'+c[3]);
}

// et pour le fun, je change la couleur à remplacer toutes les 5 secondes !
// c'est juste du Javascript, rien à voir avec p5js
var myColors = [
	[150, 160, 190],
	[255, 214, 69],
	[227, 137, 118],
	[170, 181, 114]
];
var rank = 0;
setInterval(function(){
	rank++;
	if(rank>=myColors.length){
		// retour à la première couleur
		rank = 0;
	}
	couleurCible = myColors[rank]; // mise à jour de la couleur ciblée
}, 5000);